"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault").default;
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.createAuthApolloLink = createAuthApolloLink;
exports.createFinalLink = createFinalLink;
exports.createHttpLink = createHttpLink;
exports.createTokenLink = createTokenLink;
exports.createUpdateDataLink = createUpdateDataLink;
var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/for-each"));
var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/bind"));
var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/map"));
var _client = require("@apollo/client");
var _context3 = require("@apollo/client/link/context");
var _printer = require("graphql/language/printer");
function createHttpLink(uri, httpLinkConfig, cookieHeader) {
  const headers = {};
  if (cookieHeader) {
    headers.cookie = cookieHeader;
  }
  return new _client.HttpLink({
    uri,
    credentials: 'include',
    ...httpLinkConfig,
    headers
  });
}
function enhanceError(operation, error) {
  const {
    operationName,
    query,
    variables
  } = operation;
  error.__RedwoodEnhancedError = {
    operationName,
    operationKind: query?.kind.toString(),
    variables,
    query: query && (0, _printer.print)(query)
  };
  return error;
}
function createUpdateDataLink() {
  return new _client.ApolloLink((operation, forward) => {
    return new _client.Observable(observer => {
      var _context2;
      forward(operation).subscribe({
        next(result) {
          if (result.errors) {
            var _context;
            (0, _forEach.default)(_context = result.errors).call(_context, error => {
              enhanceError(operation, error);
            });
          }
          observer.next(result);
        },
        error(error) {
          observer.error(enhanceError(operation, error));
        },
        complete: (0, _bind.default)(_context2 = observer.complete).call(_context2, observer)
      });
    });
  });
}
function createAuthApolloLink(authProviderType, headers) {
  return new _client.ApolloLink((operation, forward) => {
    const {
      token
    } = operation.getContext();
    // Only add auth headers when there's a token. `token` is `null` when `!isAuthenticated`.
    const authHeaders = token ? {
      'auth-provider': authProviderType,
      authorization: `Bearer ${token}`
    } : {};
    operation.setContext(() => ({
      headers: {
        ...operation.getContext().headers,
        ...headers,
        // Duped auth headers, because we may remove the `FetchConfigProvider` at a later date.
        ...authHeaders
      }
    }));
    return forward(operation);
  });
}
function createTokenLink(getToken) {
  return (0, _context3.setContext)(async () => {
    const token = await getToken();
    return {
      token
    };
  });
}
function createFinalLink(_ref) {
  let {
    userConfiguredLink,
    defaultLinks
  } = _ref;
  if (userConfiguredLink) {
    if (typeof userConfiguredLink === 'function') {
      return userConfiguredLink(defaultLinks);
    } else {
      return userConfiguredLink;
    }
  }
  return _client.ApolloLink.from((0, _map.default)(defaultLinks).call(defaultLinks, l => l.link));
}

// ~~~ Types ~~~

// DummyLink is needed to prevent circular dependencies when defining
// RedwoodApolloLinkName
// (Just replace DummyLink with RedwoodApolloLink in the InferredLinkName type
// helper and you'll see what I mean)