/***
 *
 * This is a lift and shift of the original ApolloProvider
 * but with suspense specific bits. Look for @MARK to find bits I've changed
 *
 * Done this way, to avoid making changes breaking on main, due to the experimental-nextjs import
 * Eventually we will have one ApolloProvider, not multiple.
 */
/// <reference types="react" />
import type { ApolloCache, ApolloClientOptions, ApolloLink, HttpOptions, InMemoryCacheConfig, setLogVerbosity } from '@apollo/client';
import type { UseAuth } from '@redwoodjs/auth';
import './typeOverride';
import type { RedwoodApolloLink, RedwoodApolloLinkFactory, RedwoodApolloLinkName, RedwoodApolloLinks } from './links';
export type ApolloClientCacheConfig = InMemoryCacheConfig;
export type { RedwoodApolloLink, RedwoodApolloLinkFactory, RedwoodApolloLinkName, RedwoodApolloLinks, };
export type GraphQLClientConfigProp = Omit<ApolloClientOptions<unknown>, 'cache' | 'link'> & {
    cache?: ApolloCache<unknown>;
    /**
     * Configuration for Apollo Client's `InMemoryCache`.
     * See https://www.apollographql.com/docs/react/caching/cache-configuration/.
     */
    cacheConfig?: ApolloClientCacheConfig;
    /**
     * Configuration for the terminating `HttpLink`.
     * See https://www.apollographql.com/docs/react/api/link/apollo-link-http/#httplink-constructor-options.
     *
     * For example, you can use this prop to set the credentials policy so that cookies can be sent to other domains:
     *
     * ```js
     * <RedwoodApolloProvider graphQLClientConfig={{
     *   httpLinkConfig: { credentials: 'include' }
     * }}>
     * ```
     */
    httpLinkConfig?: HttpOptions;
    /**
     * Extend or overwrite `RedwoodApolloProvider`'s Apollo Link.
     *
     * To overwrite Redwood's Apollo Link, just provide your own `ApolloLink`.
     *
     * To extend Redwood's Apollo Link, provide a function—it'll get passed an array of Redwood's Apollo Links
     * which are objects with a name and link property:
     *
     * ```js
     * const link = (redwoodApolloLinks) => {
     *   const consoleLink = new ApolloLink((operation, forward) => {
     *     console.log(operation.operationName)
     *     return forward(operation)
     *   })
     *
     *   return ApolloLink.from([consoleLink, ...redwoodApolloLinks.map(({ link }) => link)])
     * }
     * ```
     *
     * If you do this, there's a few things you should keep in mind:
     * - your function should return a single link (e.g., using `ApolloLink.from`; see https://www.apollographql.com/docs/react/api/link/introduction/#additive-composition)
     * - the `HttpLink` should come last (https://www.apollographql.com/docs/react/api/link/introduction/#the-terminating-link)
     */
    link?: ApolloLink | RedwoodApolloLinkFactory;
};
export declare const RedwoodApolloProvider: React.FunctionComponent<{
    graphQLClientConfig?: GraphQLClientConfigProp;
    useAuth?: UseAuth;
    logLevel?: ReturnType<typeof setLogVerbosity>;
    children: React.ReactNode;
}>;
//# sourceMappingURL=suspense.d.ts.map