"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault").default;
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.RedwoodApolloProvider = void 0;
var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/filter"));
var _globalThis2 = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/global-this"));
var _react = require("react");
var _client = require("@apollo/client");
var _ssr = require("@apollo/experimental-nextjs-app-support/ssr");
var _auth = require("@redwoodjs/auth");
require("./typeOverride");
var _FetchConfigProvider = require("../components/FetchConfigProvider");
var _GraphQLHooksProvider = require("../components/GraphQLHooksProvider");
var _links = require("./links");
var _jsxRuntime = require("react/jsx-runtime");
/***
 *
 * This is a lift and shift of the original ApolloProvider
 * but with suspense specific bits. Look for @MARK to find bits I've changed
 *
 * Done this way, to avoid making changes breaking on main, due to the experimental-nextjs import
 * Eventually we will have one ApolloProvider, not multiple.
 */

const ApolloProviderWithFetchConfig = _ref => {
  var _context;
  let {
    config,
    children,
    logLevel,
    useAuth = _auth.useNoAuth
  } = _ref;
  // Should they run into it, this helps users with the "Cannot render cell; GraphQL success but data is null" error.
  // See https://github.com/redwoodjs/redwood/issues/2473.
  (0, _client.setLogVerbosity)(logLevel);
  const {
    uri,
    headers
  } = (0, _FetchConfigProvider.useFetchConfig)();
  const {
    getToken,
    type: authProviderType
  } = useAuth();
  const isDev = process.env.NODE_ENV === 'development';
  const serverAuthState = (0, _react.useContext)(_auth.ServerAuthContext);
  const getGraphqlUrl = () => {
    // @NOTE: This comes from packages/vite/src/streaming/registerGlobals.ts
    // this needs to be an absolute url, as relative urls cannot be used in SSR
    // @TODO (STREAMING): Should this be a new config value in Redwood.toml?
    // How do we know what the absolute url is in production?
    // Possible solution: https://www.apollographql.com/docs/react/api/link/apollo-link-schema/

    return typeof window === 'undefined' ? RWJS_ENV.RWJS_EXP_SSR_GRAPHQL_ENDPOINT : uri;
  };
  const {
    httpLinkConfig,
    link: userPassedLink,
    ...otherConfig
  } = config !== null && config !== void 0 ? config : {};

  // We use this object, because that's the shape of what we pass to the config.link factory
  const redwoodApolloLinks = (0, _filter.default)(_context = [
  // @MARK REMOVE: We will not need these for cookie based auth ~~~~
  {
    name: 'withToken',
    link: (0, _links.createTokenLink)(getToken)
  }, {
    name: 'authMiddleware',
    link: (0, _links.createAuthApolloLink)(authProviderType, headers)
  },
  // ~~~~ @END REMOVE ~~~~
  isDev && {
    name: 'enhanceErrorLink',
    link: (0, _links.createUpdateDataLink)()
  }, {
    name: 'httpLink',
    link: (0, _links.createHttpLink)(getGraphqlUrl(), httpLinkConfig, serverAuthState?.cookieHeader)
  }]).call(_context, link => !!link);
  function makeClient() {
    // @MARK use special Apollo client
    return new _ssr.NextSSRApolloClient({
      link: (0, _links.createFinalLink)({
        userConfiguredLink: userPassedLink,
        defaultLinks: redwoodApolloLinks
      }),
      ...otherConfig
    });
  }
  return /*#__PURE__*/(0, _jsxRuntime.jsx)(_ssr.ApolloNextAppProvider, {
    makeClient: makeClient,
    children: children
  });
};
const RedwoodApolloProvider = _ref2 => {
  var _globalThis$__REDWOOD;
  let {
    graphQLClientConfig,
    useAuth = _auth.useNoAuth,
    logLevel = 'debug',
    children
  } = _ref2;
  // Since Apollo Client gets re-instantiated on auth changes,
  // we have to instantiate `InMemoryCache` here, so that it doesn't get wiped.
  const {
    cacheConfig,
    ...config
  } = graphQLClientConfig !== null && graphQLClientConfig !== void 0 ? graphQLClientConfig : {};

  // @MARK we need this special cache
  const cache = new _ssr.NextSSRInMemoryCache(cacheConfig).restore((_globalThis$__REDWOOD = _globalThis2.default?.__REDWOOD__APOLLO_STATE) !== null && _globalThis$__REDWOOD !== void 0 ? _globalThis$__REDWOOD : {});
  return /*#__PURE__*/(0, _jsxRuntime.jsx)(_FetchConfigProvider.FetchConfigProvider, {
    useAuth: useAuth,
    children: /*#__PURE__*/(0, _jsxRuntime.jsx)(ApolloProviderWithFetchConfig
    // This order so that the user can still completely overwrite the cache.
    , {
      config: {
        cache,
        ...config
      },
      useAuth: useAuth,
      logLevel: logLevel,
      children: /*#__PURE__*/(0, _jsxRuntime.jsx)(_GraphQLHooksProvider.GraphQLHooksProvider, {
        useQuery: _ssr.useQuery,
        useMutation: _client.useMutation,
        useSubscription: _client.useSubscription,
        useSuspenseQuery: _ssr.useSuspenseQuery,
        useBackgroundQuery: _ssr.useBackgroundQuery,
        useReadQuery: _ssr.useReadQuery,
        children: children
      })
    })
  });
};
exports.RedwoodApolloProvider = RedwoodApolloProvider;