"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault").default;
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.useBackgroundQuery = exports.GraphQLHooksProvider = exports.GraphQLHooksContext = void 0;
exports.useMutation = useMutation;
exports.useQuery = useQuery;
exports.useReadQuery = void 0;
exports.useSubscription = useSubscription;
exports.useSuspenseQuery = useSuspenseQuery;
var _react = _interopRequireDefault(require("react"));
var _jsxRuntime = require("react/jsx-runtime");
/**
 * @NOTE
 * The types QueryOperationResult, MutationOperationResult, SubscriptionOperationResult, and SuspenseQueryOperationResult
 * are overridden in packages/web/src/apollo/typeOverride.ts. This was originally so that you could bring your own gql client.
 *
 * The default (empty) types are defined in packages/web/src/global.web-auto-imports.ts
 *
 * Do not import types for hooks directly from Apollo here, unless it is an Apollo specific hook.
 */

const GraphQLHooksContext = exports.GraphQLHooksContext = _react.default.createContext({
  useQuery: () => {
    throw new Error('You must register a useQuery hook via the `GraphQLHooksProvider`');
  },
  useMutation: () => {
    throw new Error('You must register a useMutation hook via the `GraphQLHooksProvider`');
  },
  useSubscription: () => {
    throw new Error('You must register a useSubscription hook via the `GraphQLHooksProvider`');
  },
  useSuspenseQuery: () => {
    throw new Error('You must register a useSuspenseQuery hook via the `GraphQLHooksProvider`.');
  },
  //  These are apollo specific hooks!
  useBackgroundQuery: () => {
    throw new Error('You must register a useBackgroundQuery hook via the `GraphQLHooksProvider`.');
  },
  useReadQuery: () => {
    throw new Error('You must register a useReadQuery hook via the `GraphQLHooksProvider`.');
  }
});
/**
 * GraphQLHooksProvider stores standard `useQuery` and `useMutation` hooks for Redwood
 * that can be mapped to your GraphQL library of choice's own `useQuery`
 * and `useMutation` implementation.
 *
 * @todo Let the user pass in the additional type for options.
 */
const GraphQLHooksProvider = _ref => {
  let {
    useQuery,
    useMutation,
    useSubscription,
    useSuspenseQuery,
    useBackgroundQuery,
    useReadQuery,
    children
  } = _ref;
  return /*#__PURE__*/(0, _jsxRuntime.jsx)(GraphQLHooksContext.Provider, {
    value: {
      useQuery,
      useMutation,
      useSubscription,
      useSuspenseQuery,
      useBackgroundQuery,
      useReadQuery
    },
    children: children
  });
};
exports.GraphQLHooksProvider = GraphQLHooksProvider;
function useQuery(query, options) {
  return _react.default.useContext(GraphQLHooksContext).useQuery(query, options);
}
function useMutation(mutation, options) {
  return _react.default.useContext(GraphQLHooksContext).useMutation(mutation, options);
}
function useSubscription(query, options) {
  return _react.default.useContext(GraphQLHooksContext).useSubscription(query, options);
}
function useSuspenseQuery(query, options) {
  return _react.default.useContext(GraphQLHooksContext).useSuspenseQuery(query, options);
}
const useBackgroundQuery = function () {
  // @TODO something about the apollo types here mean I need to override the return type
  return _react.default.useContext(GraphQLHooksContext).useBackgroundQuery(...arguments);
};
exports.useBackgroundQuery = useBackgroundQuery;
const useReadQuery = function () {
  return _react.default.useContext(GraphQLHooksContext).useReadQuery(...arguments);
};
exports.useReadQuery = useReadQuery;