"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
var _interopRequireWildcard = require("@babel/runtime-corejs3/helpers/interopRequireWildcard").default;
var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault").default;
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.createInjector = exports.ServerInjectedHtml = exports.ServerHtmlProvider = exports.ServerHtmlContext = void 0;
exports.useServerInsertedHTML = useServerInsertedHTML;
var _set = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/set"));
var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/map"));
var _react = _interopRequireWildcard(require("react"));
var _jsxRuntime = require("react/jsx-runtime");
/**
 *
 * Inspired by Next's useServerInsertedHTML, originally designed for CSS-in-JS
 * for now it seems the only way to inject html with streaming is to use a context
 *
 * We use this for <head> tags, and for apollo cache hydration
 *
 * Until https://github.com/reactjs/rfcs/pull/219 makes it into react
 *
 */

const ServerHtmlContext = exports.ServerHtmlContext = /*#__PURE__*/_react.default.createContext(null);

/**
 *
 *  Use this factory, once per request.
 *  This is to ensure that injectionState is isolated to the request
 *  and not shared between requests.
 */
const createInjector = () => {
  const injectionState = new _set.default([]);
  const injectToPage = renderCallback => {
    injectionState.add(renderCallback);
  };
  return {
    injectToPage,
    injectionState
  };
};

// @NOTE do not instatiate the provider value here, so that we can ensure
// context isolation. This is done in streamHelpers currently,
// using the createInjector factory, once per request
exports.createInjector = createInjector;
const ServerHtmlProvider = exports.ServerHtmlProvider = ServerHtmlContext.Provider;
const ServerInjectedHtml = _ref => {
  let {
    injectionState
  } = _ref;
  const serverInsertedHtml = [];
  for (const callback of injectionState) {
    serverInsertedHtml.push(callback());

    // Remove it from the set so its not rendered again
    injectionState.delete(callback);
  }
  const fragmentId = (0, _react.useId)();
  return (0, _map.default)(serverInsertedHtml).call(serverInsertedHtml, (html, i) => {
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_react.Fragment, {
      children: html
    }, `rw-server-inserted-${fragmentId}-${i}`);
  });
};

// Exactly the same as Next's useServerInsertedHTML
exports.ServerInjectedHtml = ServerInjectedHtml;
function useServerInsertedHTML(callback) {
  const addInsertedServerHTMLCallback = (0, _react.useContext)(ServerHtmlContext);

  // Should have no effects on client where there's no flush effects provider
  if (addInsertedServerHTMLCallback) {
    addInsertedServerHTMLCallback(callback);
  }
}