"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.createServerCell = createServerCell;
var _react = require("react");
var _isCellEmpty = require("./isCellEmpty");
var _jsxRuntime = require("react/jsx-runtime");
/// <reference types="react/experimental" />

// Class components are not supported on the server
// https://nextjs.org/docs/app/building-your-application/rendering/composition-patterns#when-to-use-server-and-client-components
// Consider https://github.com/bvaughn/react-error-boundary
// import { CellErrorBoundary } from './CellErrorBoundary'

// TODO(RSC): Clean this type up and consider moving to cellTypes

function createServerCell(createCellProps) {
  const {
    data: dataFn,
    isEmpty = _isCellEmpty.isDataEmpty,
    Loading,
    Failure,
    Empty,
    Success,
    displayName = 'Cell'
  } = createCellProps;
  async function SuspendingSuccess(props) {
    // Right now, Cells don't render `children`
    const {
      children: _,
      ...variables
    } = props;
    const FailureComponent = _ref => {
      let {
        error
      } = _ref;
      if (!Failure) {
        // So that it bubbles up to the nearest error boundary
        throw error;
      }
      const queryResultWithRefetch = {
        refetch: variables => {
          // TODO (RSC): How do we refresh the page with new data?
          return dataFn(variables);
        }
      };
      return /*#__PURE__*/(0, _jsxRuntime.jsx)(Failure, {
        error: error,
        queryResult: queryResultWithRefetch
      });
    };
    try {
      const data = await dataFn(variables);
      if (isEmpty(data, {
        isDataEmpty: _isCellEmpty.isDataEmpty
      }) && Empty) {
        return /*#__PURE__*/(0, _jsxRuntime.jsx)(Empty, {
          ...props,
          ...data
        });
      }
      return /*#__PURE__*/(0, _jsxRuntime.jsx)(Success, {
        ...data,
        ...props
      });
    } catch (error) {
      return /*#__PURE__*/(0, _jsxRuntime.jsx)(FailureComponent, {
        error: error
      });
    }
  }
  SuspendingSuccess.displayName = displayName;

  // @NOTE: We are returning a HoC here!
  return props => {
    const wrapInSuspenseIfLoadingPresent = (suspendingSuccessElement, LoadingComponent) => {
      if (!LoadingComponent) {
        return suspendingSuccessElement;
      }
      return /*#__PURE__*/(0, _jsxRuntime.jsx)(_react.Suspense, {
        fallback: /*#__PURE__*/(0, _jsxRuntime.jsx)(LoadingComponent, {
          ...props
        }),
        children: suspendingSuccessElement
      });
    };
    return (
      /*#__PURE__*/
      // TODO(RSC): Do we always want a client side error boundary? If so, this
      // is where we'd add it
      // <CellErrorBoundary renderFallback={FailureComponent}>
      (0, _jsxRuntime.jsx)(_jsxRuntime.Fragment, {
        children: wrapInSuspenseIfLoadingPresent( /*#__PURE__*/(0, _jsxRuntime.jsx)(SuspendingSuccess, {
          ...props
        }), Loading)
      })
      // </CellErrorBoundary>
    );
  };
}