"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.createSuspendingCell = createSuspendingCell;
var _react = require("react");
var _client = require("@apollo/client");
var _GraphQLHooksProvider = require("../GraphQLHooksProvider");
var _CellErrorBoundary = require("./CellErrorBoundary");
var _isCellEmpty = require("./isCellEmpty");
var _jsxRuntime = require("react/jsx-runtime");
/**
 * This is part of how we let users swap out their GraphQL client while staying compatible with Cells.
 */

/**
 * Creates a Cell ~~ with Apollo Client only ~~
 * using the hooks useBackgroundQuery and useReadQuery
 *
 */
function createSuspendingCell(createCellProps) {
  const {
    QUERY,
    beforeQuery = props => ({
      // By default, we assume that the props are the gql-variables.
      variables: props,
      /**
       * We're duplicating these props here due to a suspected bug in Apollo Client v3.5.4
       * (it doesn't seem to be respecting `defaultOptions` in `RedwoodApolloProvider`.)
       *
       * @see {@link https://github.com/apollographql/apollo-client/issues/9105}
       */
      fetchPolicy: 'cache-and-network',
      notifyOnNetworkStatusChange: true
    }),
    afterQuery = data => ({
      ...data
    }),
    isEmpty = _isCellEmpty.isDataEmpty,
    Loading,
    Failure,
    Empty,
    Success,
    displayName = 'Cell'
  } = createCellProps;
  function SuspendingSuccess(props) {
    const {
      queryRef,
      suspenseQueryResult,
      userProps
    } = props;
    const {
      data,
      networkStatus
    } = (0, _GraphQLHooksProvider.useReadQuery)(queryRef);
    const afterQueryData = afterQuery(data);
    const queryResultWithNetworkStatus = {
      ...suspenseQueryResult,
      networkStatus
    };
    if (isEmpty(data, {
      isDataEmpty: _isCellEmpty.isDataEmpty
    }) && Empty) {
      return /*#__PURE__*/(0, _jsxRuntime.jsx)(Empty, {
        ...userProps,
        ...afterQueryData,
        queryResult: queryResultWithNetworkStatus
      });
    }
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(Success, {
      ...afterQueryData,
      ...userProps,
      queryResult: queryResultWithNetworkStatus
    });
  }
  SuspendingSuccess.displayName = displayName;

  // @NOTE: Note that we are returning a HoC here!
  return props => {
    /**
     * Right now, Cells don't render `children`.
     */
    const {
      children: _,
      ...variables
    } = props;
    const options = beforeQuery(variables);
    const query = typeof QUERY === 'function' ? QUERY(options) : QUERY;
    const [queryRef, other] = (0, _GraphQLHooksProvider.useBackgroundQuery)(query, options);
    const client = (0, _client.useApolloClient)();
    const suspenseQueryResult = {
      client,
      ...other,
      called: !!queryRef
    };

    // @TODO(STREAMING) removed prerender handling here
    // Until we decide how/if we do prerendering

    const FailureComponent = _ref => {
      let {
        error,
        resetErrorBoundary
      } = _ref;
      if (!Failure) {
        // So that it bubbles up to the nearest error boundary
        throw error;
      }
      const queryResultWithErrorReset = {
        ...suspenseQueryResult,
        refetch: variables => {
          resetErrorBoundary();
          return suspenseQueryResult.refetch?.(variables);
        }
      };
      return /*#__PURE__*/(0, _jsxRuntime.jsx)(Failure, {
        error: error,
        errorCode: error?.graphQLErrors?.[0]?.extensions?.['code'],
        queryResult: queryResultWithErrorReset
      });
    };
    const wrapInSuspenseIfLoadingPresent = (suspendingSuccessElement, LoadingComponent) => {
      if (!LoadingComponent) {
        return suspendingSuccessElement;
      }
      return /*#__PURE__*/(0, _jsxRuntime.jsx)(_react.Suspense, {
        fallback: /*#__PURE__*/(0, _jsxRuntime.jsx)(LoadingComponent, {
          ...props,
          queryResult: suspenseQueryResult
        }),
        children: suspendingSuccessElement
      });
    };
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_CellErrorBoundary.CellErrorBoundary, {
      renderFallback: FailureComponent,
      children: wrapInSuspenseIfLoadingPresent( /*#__PURE__*/(0, _jsxRuntime.jsx)(SuspendingSuccess, {
        userProps: props,
        queryRef: queryRef,
        suspenseQueryResult: suspenseQueryResult
      }), Loading)
    });
  };
}