"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.useDocumentReady = void 0;
var _react = require("react");
/**
 *
 * Use this hook to run a function when the document is ready.
 *
 * Useful to run things on ~first~ hard render only. Note that this will not fire on soft renders
 *
 * @param callback Any function you want to run, when document is ready
 * @returns
 */
const useDocumentReady = callback => {
  return (0, _react.useEffect)(() => {
    const handler = _e => {
      // This event fires for "interactive" and "complete"
      if (document.readyState === 'complete') {
        callback();
      }
    };
    document.addEventListener('readystatechange', handler);
    return () => {
      document.removeEventListener('readystatechange', handler);
    };
  }, [callback]);
};
exports.useDocumentReady = useDocumentReady;